@@ -28,6 +28,33 @@
 
 /*
   $Log: templates.h,v $
+ * Revision 1.15  1998/03/09  17:15:33  ewc
+ * New NP_getSlice() function in _Forany template.
+ *
+ * Revision 1.14  1998/03/05  11:20:21  sll
+ * Backoff previous change to Sequence_Array_Var.
+ *
+  Revision 1.13  1998/03/04 15:07:50  ewc
+  Small bug fix to Sequence_Var types
+
+ * Revision 1.12  1998/03/04  14:48:34  sll
+ * Disambiguous reference to NP_data() by adding the _CORBA_Sequence<T>::
+ * prefix.
+ *
+ * Revision 1.11  1998/01/27  16:11:06  ewc
+ * Added support for type any.
+ *
+  Revision 1.10  1997/12/23 19:37:57  sll
+  Removed const T* conversion operator in _CORBA_Array_Var as it is causing
+  conversion ambiguity.
+  Demote workarounds for older DEC C++ compiler as the bug has been fixed.
+
+  Revision 1.9  1997/12/09 20:41:02  sll
+  Updated sequence array templates.
+
+  Revision 1.8  1997/08/21 22:22:45  sll
+  New templates for sequence of array.
+
  * Revision 1.7  1997/05/06  16:11:33  sll
  * Public release.
  *
@@ -56,7 +83,7 @@ public:
   inline _CORBA_ObjRef_Var() { pd_objref = T_Helper::_nil(); }
   inline _CORBA_ObjRef_Var(T *p) { pd_objref = p; }
   inline ~_CORBA_ObjRef_Var() { 
-#ifndef __DECCXX
+#if !defined(__DECCXX) || (__DECCXX_VER >= 50500000)
     if (!T_Helper::is_nil(pd_objref)) T_Helper::release(pd_objref);
 #else
     if (!CORBA::is_nil(pd_objref)) CORBA::release(pd_objref);
@@ -91,7 +118,7 @@ public:
   inline _CORBA_ObjRef_Member(T *p) { _ptr = p; }
   inline _CORBA_ObjRef_Member(const _CORBA_ObjRef_Member<T,T_Helper> &p);
   inline ~_CORBA_ObjRef_Member() { 
-#ifndef __DECCXX
+#if !defined(__DECCXX) || (__DECCXX_VER >= 50500000)
     if (!T_Helper::is_nil(_ptr)) T_Helper::release(_ptr);
 #else
     if (!CORBA::is_nil(_ptr)) CORBA::release(_ptr);
@@ -252,9 +279,9 @@ public:
   inline ~_CORBA_Sequence_Var() {  if (pd_data) delete pd_data; }
   inline _CORBA_Sequence_Var<T,ElmType> &operator= (T* p);
   inline _CORBA_Sequence_Var<T,ElmType> &operator= (const _CORBA_Sequence_Var<T,ElmType> &p);
-  inline ElmType &operator[] (_CORBA_ULong index) { return (pd_data->NP_data())[index]; }
+  inline ElmType &operator[] (_CORBA_ULong index) { return (pd_data->_CORBA_Sequence<ElmType>::NP_data())[index]; }
   inline const ElmType &operator[] (_CORBA_ULong index) const {
-    return (pd_data->NP_data())[index];
+    return (pd_data->_CORBA_Sequence<ElmType>::NP_data())[index];
   }
   inline T* operator->() const { return (T*)pd_data; }
 
@@ -281,6 +308,45 @@ private:
   T* pd_data;
 };
 
+template <class T,class T_slice>
+class _CORBA_Sequence_Array_Var {
+public:
+  typedef T* ptr_t;
+  inline _CORBA_Sequence_Array_Var() { pd_data = 0; }
+  inline _CORBA_Sequence_Array_Var(T* p) { pd_data = p; }
+  inline _CORBA_Sequence_Array_Var(const _CORBA_Sequence_Array_Var<T,T_slice> &p);
+  inline ~_CORBA_Sequence_Array_Var() {  if (pd_data) delete pd_data; }
+  inline _CORBA_Sequence_Array_Var<T,T_slice> &operator= (T* p);
+  inline _CORBA_Sequence_Array_Var<T,T_slice> &operator= (const _CORBA_Sequence_Array_Var<T,T_slice> &p);
+  inline T_slice* operator[] (_CORBA_ULong index) { return (T_slice*)((pd_data->NP_data())[index]); }
+  inline const T_slice* operator[] (_CORBA_ULong index) const {
+    return (const T_slice*)((pd_data->NP_data())[index]);
+  }
+  inline T* operator->() const { return (T*)pd_data; }
+
+#if defined(__GNUG__) && __GNUG__ == 2 && __GNUC_MINOR__ == 7
+  inline operator T& () const { return *pd_data; }
+#else
+  inline operator const T& () const { return *pd_data; }
+  inline operator T& () { return *pd_data; }
+#endif
+  // This conversion operator is necessary to support the implicit conversion
+  // when this var type is used as the IN or INOUT argument of an operation.
+
+  // The following coversion operators are needed to support the casting
+  // of this var type to a const T* or a T*. The CORBA spec. doesn't say
+  // these castings must be supported so they are deliberately left out.
+  // In fact, the operator->() can always be used to get to the T*.
+  //
+  // inline operator const T* () const { return pd_data; }
+  // inline operator T* () { return pd_data; }
+
+  friend class _CORBA_Sequence_OUT_arg<T,_CORBA_Sequence_Array_Var<T,T_slice> >;
+
+private:
+  T* pd_data;
+};
+
 
 template <class T, class T_var>
 class _CORBA_Sequence_OUT_arg {
@@ -307,9 +373,13 @@ public:
   inline _CORBA_Array_Var<T_Helper,T> &operator= (T* p);
   inline _CORBA_Array_Var<T_Helper,T> &operator= (const _CORBA_Array_Var<T_Helper,T>& p);
   inline T& operator[] (_CORBA_ULong index) { return *(pd_data + index); }
-  inline const T& operator[] (_CORBA_ULong index) const { return *(pd_data + index);  }
+  inline const T& operator[] (_CORBA_ULong index) const { return *( (const T*) (pd_data + index));  }
   inline operator T* () const { return pd_data; }
-  inline operator const T* () const { return pd_data; }
+  // Define the const T* operator() causes conversion operator ambiguity with 
+  // some compilers. Should be alright to leave this operator out. If not,
+  // reinstate it and #ifdef it with the right compiler specific macro.
+  //
+  //  inline operator const T* () const { return (const T*) pd_data; }
 
   friend class _CORBA_Array_OUT_arg<T, _CORBA_Array_Var<T_Helper,T> >;
 
@@ -338,10 +408,13 @@ public:
   inline ~_CORBA_Array_Forany() {
     // does not delete the storage of the underlying array
   }
+  inline _CORBA_Array_Forany<T_Helper,T> &operator= (T* p);
   inline T& operator[] (_CORBA_ULong index) { return *(pd_data + index); }
-  inline const T& operator[] (_CORBA_ULong index) const { return *(pd_data + index); }
+  inline const T& operator[] (_CORBA_ULong index) const { return *( (const T*) (pd_data + index)); }
   inline operator T* () const { return pd_data; }
-  inline operator const T* () const { return pd_data; }
+  inline operator const T* () const { return (const T*) pd_data; }
+
+  inline T* NP_getSlice() const { return pd_data; }
 private:
   T* pd_data;
 };
@@ -369,6 +442,28 @@ _CORBA_Bounded_Sequence<T,max>::length(_CORBA_ULong len)
   return;
 }
 
+template <class T,class T_slice,class Telm,int dimension,int max>
+inline 
+_CORBA_Bounded_Sequence_Array<T,T_slice,Telm,dimension,max>&
+_CORBA_Bounded_Sequence_Array<T,T_slice,Telm,dimension,max>::operator= (const _CORBA_Bounded_Sequence_Array<T,T_slice,Telm,dimension,max> &s)
+{
+  _CORBA_Sequence_Array<T,T_slice,Telm,dimension>::operator= (s);
+  return *this;
+}
+
+template <class T,class T_slice,class Telm,int dimension,int max>
+inline 
+void
+_CORBA_Bounded_Sequence_Array<T,T_slice,Telm,dimension,max>::length(_CORBA_ULong len)
+{
+  if (len > max) {
+    _CORBA_bound_check_error();
+    // never reach here
+  }
+  _CORBA_Sequence_Array<T,T_slice,Telm,dimension>::length(len);
+  return;
+}
+
 
 template <class T,int max,int elmSize, int elmAlignment>
 inline
@@ -401,6 +496,37 @@ _CORBA_Bounded_Sequence_w_FixSizeElement<T,max,elmSize,elmAlignment>::length(_CO
   return;
 }
 
+template <class T,class T_slice,class Telm,int dimension,int max,int elmSize, int elmAlignment>
+inline
+_CORBA_Bounded_Sequence_Array_w_FixSizeElement<T,T_slice,Telm,dimension,max,elmSize,elmAlignment>&
+_CORBA_Bounded_Sequence_Array_w_FixSizeElement<T,T_slice,Telm,dimension,max,elmSize,elmAlignment>::operator=(const 
+_CORBA_Bounded_Sequence_Array_w_FixSizeElement<T,T_slice,Telm,dimension,max,elmSize,elmAlignment>& s) 
+{
+  _CORBA_Sequence_Array<T,T_slice,Telm,dimension>::operator=(s);
+  return *this;
+}
+
+template <class T,class T_slice,class Telm,int dimension,int max,int elmSize, int elmAlignment>
+inline
+_CORBA_ULong
+_CORBA_Bounded_Sequence_Array_w_FixSizeElement<T,T_slice,Telm,dimension,max,elmSize,elmAlignment>::length() const
+{ 
+  return _CORBA_Sequence_Array<T,T_slice,Telm,dimension>::length();
+}
+
+template <class T,class T_slice,class Telm,int dimension,int max,int elmSize, int elmAlignment>
+inline
+void
+_CORBA_Bounded_Sequence_Array_w_FixSizeElement<T,T_slice,Telm,dimension,max,elmSize,elmAlignment>::length(_CORBA_ULong len)
+{
+  if (len > max) {
+    _CORBA_bound_check_error();
+    // never reach here
+  }
+  _CORBA_Sequence_Array<T,T_slice,Telm,dimension>::length(len);
+  return;
+}
+
 
 template <class T,class T_Helper>
 inline
@@ -612,6 +738,28 @@ _CORBA_ConstrType_Variable_Var<T>::operator= (const _CORBA_ConstrType_Variable_V
   return *this;
 }
 
+template <class T,class ElmType>
+inline
+_CORBA_Sequence_Var<T,ElmType> &
+_CORBA_Sequence_Var<T,ElmType>::operator= (const _CORBA_Sequence_Var<T,ElmType> &p)
+{
+  if (p.pd_data) {
+    if (!pd_data) {
+      pd_data = new T;
+      if (!pd_data) {
+	_CORBA_new_operator_return_null();
+	// never reach here
+      }
+    }
+    *pd_data = *p.pd_data;
+  }
+  else {
+    if (pd_data) delete pd_data;
+    pd_data = 0;
+  }
+  return *this;
+}
+
 template <class T,class ElmType>
 inline 
 _CORBA_Sequence_Var<T,ElmType>::_CORBA_Sequence_Var(const _CORBA_Sequence_Var<T,ElmType> &p) 
@@ -640,10 +788,10 @@ _CORBA_Sequence_Var<T,ElmType>::operator= (T* p)
   return *this;
 }
 
-template <class T,class ElmType>
+template <class T,class T_slice>
 inline
-_CORBA_Sequence_Var<T,ElmType> &
-_CORBA_Sequence_Var<T,ElmType>::operator= (const _CORBA_Sequence_Var<T,ElmType> &p)
+_CORBA_Sequence_Array_Var<T,T_slice> &
+_CORBA_Sequence_Array_Var<T,T_slice>::operator= (const _CORBA_Sequence_Array_Var<T,T_slice> &p)
 {
   if (p.pd_data) {
     if (!pd_data) {
@@ -662,6 +810,35 @@ _CORBA_Sequence_Var<T,ElmType>::operator= (const _CORBA_Sequence_Var<T,ElmType>
   return *this;
 }
 
+template <class T,class T_slice>
+inline 
+_CORBA_Sequence_Array_Var<T,T_slice>::_CORBA_Sequence_Array_Var(const _CORBA_Sequence_Array_Var<T,T_slice> &p) 
+{
+  if (!p.pd_data) {
+    pd_data = 0;
+    return;
+  }
+  else {
+    pd_data = new T;
+    if (!pd_data) {
+      _CORBA_new_operator_return_null();
+      // never reach here
+    }
+    *pd_data = *p.pd_data;
+  }
+}
+
+template <class T,class T_slice>
+inline
+_CORBA_Sequence_Array_Var<T,T_slice> &
+_CORBA_Sequence_Array_Var<T,T_slice>::operator= (T* p)
+{
+  if (pd_data) delete pd_data;
+  pd_data = p;
+  return *this;
+}
+
+
 template <class T_Helper,class T>
 inline
 _CORBA_Array_Var<T_Helper,T>::_CORBA_Array_Var (const _CORBA_Array_Var<T_Helper,T >& p) 
@@ -729,6 +906,16 @@ _CORBA_Array_Forany<T_Helper,T>::_CORBA_Array_Forany (T* p,_CORBA_Boolean nocopy
   }
 }
 
+
+template <class T_Helper,class T>
+inline 
+_CORBA_Array_Forany<T_Helper,T>&
+_CORBA_Array_Forany<T_Helper,T>::operator= (T* p) 
+{
+  pd_data = p;
+  return *this;
+}
+
 template <class T_Helper,class T>
 inline 
 _CORBA_Array_Forany<T_Helper,T>::_CORBA_Array_Forany (const _CORBA_Array_Forany<T_Helper,T>& p)
