@@ -30,12 +30,16 @@
 #include <time.h>
 #include <iostream.h>
 #include <fcntl.h>
+#if defined(__VMS) && __VMS_VER < 70000000
+#include <omniVMS/unlink.hxx>
+#include <omniVms/utsname.hxx>
+#endif
 #include <NamingContext_i.h>
 #include <ObjectBinding.h>
 #include <log.h>
 #include <iomanip.h>
 
-#ifdef __NT__
+#ifdef __WIN32__
 
 #include <io.h>
 #include <winbase.h>
@@ -50,6 +54,32 @@
 
 #endif
 
+#if defined(__nextstep__)
+#include <libc.h>
+#include <sys/param.h>
+#endif
+
+#if defined(__nextstep__)
+#include <sys/param.h>
+#endif
+
+
+#ifdef _NO_STRDUP
+
+
+// we have no strdup
+static char *
+strdup (char* str)
+{
+  char *newstr;
+
+  newstr = (char *) malloc (strlen (str) + 1);
+  if (newstr)
+    strcpy (newstr, str);
+  return newstr;
+}
+#endif  // _NO_STRDUP
+
 extern void usage();
 
 
@@ -101,7 +131,7 @@ log::log(int& p) : port(p)
   checkpointNeeded = 1;
   line = 1;
 
-#ifdef __NT__
+#ifdef __WIN32__
   struct _stat sb;
 #else
   struct stat sb;
@@ -112,7 +142,7 @@ log::log(int& p) : port(p)
   if ((logdir = getenv(LOGDIR_ENV_VAR)) == NULL)
     logdir = strdup(DEFAULT_LOGDIR);
 
-#ifndef __NT__  
+#if !defined(__WIN32__) && !defined(__VMS)
   if (logdir[0] != '/') {
     cerr << ts.t() << "Error: " << LOGDIR_ENV_VAR << " (" << logdir
 	 << ") is not an absolute path name." << endl;
@@ -123,6 +153,7 @@ log::log(int& p) : port(p)
     logdir[strlen(logdir)-1] = '\0';		// strip trailing '/'
   }
 
+#ifndef _USE_GETHOSTNAME
   struct utsname un;
   if (uname(&un) < 0) {
     cerr << ts.t() << "Error: cannot get the name of this host." << endl;
@@ -133,8 +164,19 @@ log::log(int& p) : port(p)
   char* logname = new char[strlen(logdir) + strlen("/omninames-")
 			   + strlen(un.nodename) + 1];
   sprintf(logname, "%s/omninames-%s", logdir, un.nodename);
-
 #else
+  char hostname[MAXHOSTNAMELEN+1];
+
+  if (gethostname(hostname, MAXHOSTNAMELEN) < 0) {
+    cerr << ts.t() << "Error: cannot get the name of this host." << endl;
+	
+    exit(1);
+  }
+  char* logname = new char[strlen(logdir) + strlen("/omninames-")
+			   + strlen(hostname) + 1];
+  sprintf(logname, "%s/omninames-%s", logdir, hostname);
+#endif // _USE_GETHOSTNAME
+#elif defined(__WIN32__)
 
   // Get host name:
 
@@ -151,14 +193,50 @@ log::log(int& p) : port(p)
   sprintf(logname, "%s\\omninames-%s", logdir, machineName);
   
   delete[] machineName;
+#else
+  char last(
+    logdir[strlen(logdir)-1]
+  );
+  if (last != ':' && last != ']') {
+    cerr << ts.t() << "Error: " << LOGDIR_ENV_VAR << " (" << logdir
+         << ") is not a directory name." << endl;
+    exit(1);
+  }
+
+//  if (logdir[strlen(logdir)-1] == '/') {
+//    logdir[strlen(logdir)-1] = '\0';          // strip trailing '/'
+//  }
+
+  struct utsname un;
+  if (uname(&un) < 0) {
+    cerr << ts.t() << "Error: cannot get the name of this host." << endl;
+        
+    exit(1);
+  }
+
+  char* logname = new char[strlen(logdir) + strlen("/omninames-")
+                           + strlen(un.nodename) + 1];
+  sprintf(logname, "%somninames-%s", logdir, un.nodename);
 #endif
 
+#ifndef __VMS
   active = new char[strlen(logname)+strlen(".log")+1];
   sprintf(active,"%s.log",logname);
   backup = new char[strlen(logname)+strlen(".bak")+1];
   sprintf(backup,"%s.bak",logname);
   checkpt = new char[strlen(logname)+strlen(".ckp")+1];
   sprintf(checkpt,"%s.ckp",logname);
+#else
+  // specify latest version:
+  active = new char[strlen(logname)+strlen(".log;")+1];
+  sprintf(active,"%s.log;",logname);
+  backup = new char[strlen(logname)+strlen(".bak;")+1];
+  sprintf(backup,"%s.bak;",logname);
+  checkpt = new char[strlen(logname)+strlen(".ckp;")+1];
+  sprintf(checkpt,"%s.ckp;",logname);
+#endif
+
+  delete [] logname;
 
   if (port != 0) {
 
@@ -188,7 +266,7 @@ log::log(int& p) : port(p)
 
     firstTime = 0;
 
-#ifdef __NT__
+#ifdef __WIN32__
     ifstream initf(active,ios::in | ios::nocreate);
 #else
     ifstream initf(active);
@@ -246,7 +324,7 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
     cerr << ts.t() << "Starting omniNames for the first time." << endl;
 
     try {
-#ifdef __NT__
+#ifdef __WIN32__
       int fd = _open(active, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
 #else
       int fd = open(active, O_WRONLY | O_CREAT | O_TRUNC | O_SYNC, 0666);
@@ -267,8 +345,8 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
       if (!logf)
 	throw IOError();
 
-// a bug on Solaris means that the fd doesn't get closed.
-#if defined(__sunos__) && (__OSVERSION__ == 5)
+// a bug in sparcworks C++ means that the fd doesn't get closed.
+#if defined(__sunos__) && defined(__SUNPRO_CC)
       if (close(fd) < 0)
 	throw IOError();
 #endif
@@ -355,10 +433,15 @@ log::init(CORBA::ORB_ptr o, CORBA::BOA_ptr b)
 
   char* p = orb->object_to_string(rootContext);
   cerr << ts.t() << "Root context is " << p << endl;
+  // Now use the backdoor to tell the bootstrap agent in this
+  // address space to return this root context in response to
+  // CORBA::InitialReferences::get("NameService");
+  _omni_set_NameService(rootContext);
+  delete p;
 
   CORBA::release(rootContext);	// dispose of the object reference
 
-#ifdef __NT__
+#ifdef __WIN32__
   int fd = _open(active, O_WRONLY | O_APPEND);
 #else
   int fd = open(active, O_WRONLY | O_APPEND | O_SYNC);
@@ -465,7 +548,7 @@ log::checkpoint(void)
 
   ofstream ckpf;
 
-#ifdef __NT__
+#ifdef __WIN32__
   int fd = _open(checkpt, O_WRONLY | O_CREAT | O_TRUNC, _S_IWRITE);
 #else
   int fd = open(checkpt, O_WRONLY | O_CREAT | O_TRUNC | O_SYNC, 0666);
@@ -499,8 +582,8 @@ log::checkpoint(void)
     if (!ckpf)
       throw IOError();
 
-// a bug on Solaris means that the fd doesn't get closed.
-#if defined(__sunos__) && (__OSVERSION__ == 5)
+// a bug in sparcworks C++ means that the fd doesn't get closed.
+#if defined(__sunos__) && defined(__SUNPRO_CC)
     if (close(fd) < 0)
       throw IOError();
 #endif
@@ -510,8 +593,8 @@ log::checkpoint(void)
     perror("I/O error writing checkpoint file");
     cerr << "Abandoning checkpoint" << endl;
     ckpf.close();
-// a bug on Solaris means that the fd doesn't get closed.
-#if defined(__sunos__) && (__OSVERSION__ == 5)
+// a bug in sparcworks C++ means that the fd doesn't get closed.
+#if defined(__sunos__) && defined(__SUNPRO_CC)
     close(fd);
 #endif
     NamingContext_i::lock.readerOut();
@@ -526,8 +609,8 @@ log::checkpoint(void)
 
   cerr << ts.t() << "Checkpointing Phase 2: Commit." << endl;
 
-// a bug on Solaris means that the fd doesn't get closed.
-#if defined(__sunos__) && (__OSVERSION__ == 5)
+// a bug in sparcworks C++ means that the fd doesn't get closed.
+#if defined(__sunos__) && defined(__SUNPRO_CC)
   close(logf.rdbuf()->fd());
 #endif
 
@@ -535,8 +618,10 @@ log::checkpoint(void)
 
   unlink(backup);
 
-#ifdef __NT__
+#if defined(__WIN32__)
   if (!CopyFile(active,backup,TRUE)) {
+#elif defined(__VMS)
+  if (rename(active, backup) < 0) {
 #else
   if (link(active,backup) < 0) {
 #endif
@@ -546,16 +631,19 @@ log::checkpoint(void)
     exit(1);
   }
 
+#ifndef __VMS
   if (unlink(active) < 0) {
     // Failure here leaves active and backup pointing to the same (old) file.
     cerr << ts.t() << "Error: failed to unlink old log file '" << active
 	 << "'." << endl;
     exit(1);
   }
+#endif
 
-
-#ifdef __NT__
+#if defined(__WIN32__)
   if (!CopyFile(checkpt,active,TRUE)) {
+#elif defined(__VMS)
+  if (rename(checkpt,active) < 0) {
 #else
   if (link(checkpt,active) < 0) {
 #endif
@@ -565,14 +653,16 @@ log::checkpoint(void)
     exit(1);
   }
 
+#ifndef __VMS
   if (unlink(checkpt) < 0) {
     // Failure here leaves active and checkpoint pointing to the same file.
     cerr << ts.t() << "Error: failed to unlink checkpoint file '" << checkpt
 	 << "'." << endl;
     exit(1);
   }
+#endif
 
-#ifdef __NT__
+#ifdef __WIN32__
   fd = _open(active, O_WRONLY | O_APPEND);
 #else
   fd = open(active, O_WRONLY | O_APPEND | O_SYNC);
@@ -867,6 +957,7 @@ log::putKey(const omniORB::objectKey& key, ostream& file)
     file << setfill('0') << setw(2) << (int)(*os)[i];
   }
   file << dec;
+  delete os;
 }
 
 
@@ -1001,7 +1092,7 @@ log::getString(char*& buf, istream& file)
       }
     }
 
-    if (p >= (buf + bufsz)) {
+    if (p == (buf + bufsz)) {
 
       // buffer is too small
 
@@ -1009,6 +1100,7 @@ log::getString(char*& buf, istream& file)
       buf = new char[bufsz+bufsz];
       memcpy(buf, obuf, bufsz);
       delete [] obuf;
+      p = buf + bufsz;
       bufsz += bufsz;
     }
   }
