@@ -62,18 +62,19 @@ trademarks or registered trademarks of Sun Microsystems, Inc.
 
  */
 
-#pragma ident "%@(#)drv_preproc.cc	1.16% %92/06/10% Sun Microsystems"
-
 /*
  * DRV_pre_proc.cc - pass an IDL file through the C preprocessor
  */
 
 #include	<idl.hh>
 #include	<idl_extern.hh>
-
 #include	<drv_private.hh>
 #include	<drv_link.hh>
 
+#ifdef HAS_pch
+#pragma hdrstop
+#endif
+
 #include	<stdio.h>
 #include	<stdlib.h>
 #include	<string.h>
@@ -92,11 +93,15 @@ trademarks or registered trademarks of Sun Microsystems, Inc.
 #endif
 #endif
 
-#if defined(__osf1__)
+#if defined(__osf1__) || defined(__VMS)
 #include	<unistd.h>		// POSIX standard types
 #include	<wait.h>		// POSIX definition of wait()
 extern "C" char * mktemp(char *);
 
+#if defined(__VMS) && __VMS_VER < 70000000
+#include <omniVms/unlink.hxx>
+#endif
+
 #endif
 
 
@@ -104,17 +109,41 @@ extern "C" char * mktemp(char *);
 #include	<sysent.h>
 #endif		// apollo
 
-#if defined(hpux) || defined(__hpux)
+#if defined(__hpux__)
 #include	<unistd.h>		// POSIX definitions
 #include	<sys/wait.h>		// POSIX definition of wait()
-#endif		// defined(hpux) || defined(__hpux)
+#endif		// defined(__hpux__)
 
-#ifdef __NT__
+#if defined(__aix__) || defined(__SINIX__)
+#include        <unistd.h>              // POSIX standard types
+#include        <sys/wait.h>            // POSIX definition of wait()
+#endif
+
+#if defined(__GLIBC__) && __GLIBC__ >= 2
+#include <unistd.h>                     // POSIX standard types
+#include <sys/wait.h>
+#endif
+
+#if defined(__nextstep__)
+#include <sys/types.h>
+typedef int pid_t;
+#endif
+
+#if defined(__irix__) 
+#include      <unistd.h>              // POSIX standard types
+#include      <wait.h>                // POSIX definition of wait()
+#endif
+
+#ifdef __WIN32__
 #include <io.h>
 #include <process.h>
 #include <sys/stat.h>
 #endif
 
+#ifndef EXIT_FAILURE
+#define EXIT_FAILURE 1
+#endif
+
 #undef	MAX_ARGLIST
 #define	MAX_ARGLIST	128
 
@@ -128,6 +157,10 @@ static  idl_bool  copy_src = I_TRUE;
 void
 DRV_cpp_new_location(char *new_loc)
 {
+#ifdef __irix__
+  char *help = strchr (new_loc, ' ');
+  if (help) *help = '\0';
+#endif
   arglist[0] = new_loc;
 }
 
@@ -137,8 +170,12 @@ DRV_cpp_new_location(char *new_loc)
 void
 DRV_cpp_putarg(char *str)
 {
+#ifdef __irix__
+  char *help = strchr (str, ' ');
+  if (help) *help = '\0';
+#endif
   if (argcount >= MAX_ARGLIST) {
-    cerr << idl_global->prog_name()
+    std::cerr << idl_global->prog_name()
          << GTDEVEL(": More than ")
 	 << MAX_ARGLIST
 	 << GTDEVEL(" arguments to preprocessor\n");
@@ -168,7 +205,6 @@ DRV_cpp_init()
   }
 
   DRV_cpp_putarg("-E");
-  DRV_cpp_putarg("-DIDL");
   DRV_cpp_putarg("-I.");
 }
 
@@ -208,16 +244,16 @@ DRV_copy_input(FILE *fin, char *fn)
   FILE	*f = fopen(fn, "w");
 
   if (f == NULL) {
-    cerr << idl_global->prog_name()
+    std::cerr << idl_global->prog_name()
 	 << GTDEVEL(": cannot open temp file ")
 	 << fn
 	 << GTDEVEL(" for writing\n");
-    exit(99);
+    exit(EXIT_FAILURE);
   }
   if (fin == NULL) {
-      cerr << idl_global->prog_name()
+      std::cerr << idl_global->prog_name()
            << GTDEVEL(": cannot open input file\n");
-      exit(99);
+      exit(EXIT_FAILURE);
   }
   while (DRV_get_line(fin))
     fprintf(f, "%s\n", drv_line);
@@ -237,9 +273,12 @@ DRV_stripped_name(char *fn)
     if (n == NULL)
 	return NULL;
     l = strlen(n);
-#ifdef __NT__
+#if defined(__WIN32__)
     for (n += l; l > 0 && *n != '\\'; l--, n--);
     if (*n == '\\') n++;
+#elif defined(__VMS)
+    for (n += l; l > 0 && *n != ']' && *n != ':'; l--, n--);
+    if (*n == ']' || *n == ':') n++;
 #else
     for (n += l; l > 0 && *n != '/'; l--, n--);
     if (*n == '/') n++;
@@ -255,7 +294,7 @@ static char	tmp_file[128];
 static char	tmp_ifile[128];
 
 
-#ifndef __NT__
+#if !defined(__WIN32__) && !defined(__VMS)
 /*
  * Pass input through preprocessor
  */
@@ -315,29 +354,29 @@ DRV_pre_proc(char *myfile)
     {
       int fd = open(tmp_file, O_WRONLY | O_CREAT | O_TRUNC, 0777);
       if (fd < 0) {
-        cerr << idl_global->prog_name()
+        std::cerr << idl_global->prog_name()
     	  << GTDEVEL(": cannot open temp file ")
  	  << tmp_file << " for writing\n";
-        exit(99);
+        exit(EXIT_FAILURE);
       }
       int result = dup2(fd, 1);
       if (result < 0) {
-        cerr << idl_global->prog_name()
+        std::cerr << idl_global->prog_name()
     	  << GTDEVEL(": temp file ")
   	  << tmp_file << " dup error\n";
-        exit(99);
+        exit(EXIT_FAILURE);
       }
       close(fd);
     }
     execvp(arglist[0], arglist);
-    cerr << idl_global->prog_name() 
+    std::cerr << idl_global->prog_name() 
          << GTDEVEL(": execvp of ")
 	 << arglist[0]
 	 << GTDEVEL(" failed\n");
-    exit(99);
+    exit(EXIT_FAILURE);
   case -1:
-    cerr << idl_global->prog_name() << GTDEVEL(": fork failed\n");
-    exit(99);
+    std::cerr << idl_global->prog_name() << GTDEVEL(": fork failed\n");
+    exit(EXIT_FAILURE);
   default:	/* Parent - wait */
 #if defined(hpux) || defined(__hpux)
     sleep(1);  // try to get around libc_r defect
@@ -346,7 +385,7 @@ DRV_pre_proc(char *myfile)
 #if defined(WIFEXITED) && defined(WEXITSTATUS)
     if (WIFEXITED(wait_status)) {
       if (WEXITSTATUS(wait_status) != 0) {
-        cerr << idl_global->prog_name()
+        std::cerr << idl_global->prog_name()
 	     << GTDEVEL(": Preprocessor returned non-zero status ")
 	     << (int) WEXITSTATUS(wait_status)
 	     << "\n";
@@ -356,7 +395,7 @@ DRV_pre_proc(char *myfile)
       }
     } else {
       // child terminated abnormally - wait_status is meaningless
-      cerr << idl_global->prog_name()
+      std::cerr << idl_global->prog_name()
 	   << GTDEVEL(": Preprocessor terminated abnormally")
 	   << "\n";
       if (copy_src) unlink(tmp_ifile);
@@ -369,7 +408,7 @@ DRV_pre_proc(char *myfile)
 #else
     if (wait_status != 0) {
 #endif	// defined(apollo) || defined(SUNOS4)
-      cerr << idl_global->prog_name()
+      std::cerr << idl_global->prog_name()
 	   << GTDEVEL(": Preprocessor returned non-zero status ")
 #if defined(apollo) || defined(SUNOS4)
 	   << wait_status.w_status
@@ -389,11 +428,11 @@ DRV_pre_proc(char *myfile)
   }
   FILE * yyin = fopen(tmp_file, "r");
   if (yyin == NULL) {
-    cerr << idl_global->prog_name()
+    std::cerr << idl_global->prog_name()
 	 << GTDEVEL(": Could not open cpp output file ")
 	 << tmp_file
 	 << "\n";
-    exit(99);
+    exit(EXIT_FAILURE);
   }
   (*DRV_FE_set_yyin)((File *) yyin);
   if (idl_global->compile_flags() & IDL_CF_ONLY_PREPROC) {
@@ -402,27 +441,27 @@ DRV_pre_proc(char *myfile)
   }
   if (copy_src) {
     if (unlink(tmp_ifile) != 0) {
-      cerr << idl_global->prog_name()
+      std::cerr << idl_global->prog_name()
 	   << GTDEVEL(": Could not remove cpp input file ")
 	   << tmp_ifile
 	   << "\n";
-      exit(99);
+      exit(EXIT_FAILURE);
     }
   }
   if (unlink(tmp_file) != 0) {
-    cerr << idl_global->prog_name()
+    std::cerr << idl_global->prog_name()
 	 << GTDEVEL(": Could not remove cpp output file ")
 	 << tmp_file
 	 << "\n";
-    exit(99);
+    exit(EXIT_FAILURE);
   }
   if (idl_global->compile_flags() & IDL_CF_ONLY_PREPROC)
     exit(0);
 }
 
-#else
+#elif defined(__WIN32__)
 
-// Windows NT Version
+// WIN 32 Version
 
 /*
  * Pass input through preprocessor
@@ -441,12 +480,12 @@ DRV_pre_proc(char *myfile)
   tmpfn = _tempnam(NULL,"idl_");
   if (tmpfn == NULL)
     {
-      cerr << "Error creating temporary filename." << endl;
+      std::cerr << "Error creating temporary filename." << std::endl;
       exit(-1);
     }
   else if (strlen(tmpfn) > 127)
     {
-      cerr << "Temporary filename too large." << endl;
+      std::cerr << "Temporary filename too large." << std::endl;
       exit(-1);
     }
   
@@ -457,12 +496,12 @@ DRV_pre_proc(char *myfile)
 
   if (tmpfn == NULL)
     {
-      cerr << "Error creating temporary filename." << endl;
+      std::cerr << "Error creating temporary filename." << std::endl;
       exit(-1);
     }
   else if (strlen(tmpfn) > 127)
     {
-      cerr << "Temporary filename too large." << endl;
+      std::cerr << "Temporary filename too large." << std::endl;
       exit(-1);
     }
   strcpy(tmp_ifile,tmpfn);
@@ -509,17 +548,17 @@ DRV_pre_proc(char *myfile)
       int fd = _open(tmp_file, _O_RDWR | _O_CREAT | _O_TRUNC, 
 			                                        _S_IWRITE);
       if (fd < 0) {
-        cerr << idl_global->prog_name()
+        std::cerr << idl_global->prog_name()
     	  << GTDEVEL(": cannot open temp file ")
  	  << tmp_file << " for writing\n";
-        exit(99);
+        exit(EXIT_FAILURE);
       }
       int result = _dup2(fd, _fileno(stdout));
       if (result < 0) {
-        cerr << idl_global->prog_name()
+        std::cerr << idl_global->prog_name()
     	  << GTDEVEL(": temp file ")
   	  << tmp_file << " dup error\n";
-        exit(99);
+        exit(EXIT_FAILURE);
       }
       _close(fd);
     }
@@ -527,42 +566,42 @@ DRV_pre_proc(char *myfile)
 			                 (const char* const*) arglist);
 	if (spawn_rc != 0)
 		{
-			cerr << idl_global->prog_name() 
+			std::cerr << idl_global->prog_name() 
 				 << GTDEVEL(": spawnvp of ")
 				 << arglist[0]
 				 << GTDEVEL(" failed\n");
-			cerr << "Preprocessor returned non-zero status " << spawn_rc << endl;
+			std::cerr << "Preprocessor returned non-zero status " << spawn_rc << std::endl;
 			
 			_close(1);
 			if (copy_src) {
 				 if (_unlink(tmp_ifile) == -1) {
-					cerr << idl_global->prog_name()
+					std::cerr << idl_global->prog_name()
 						 << GTDEVEL(": Could not remove cpp input file ")
 						 << tmp_ifile
 						 << "\n";
-					exit(99);	
+					exit(EXIT_FAILURE);	
 					}
 			 }
 
 			if (_unlink(tmp_file) == -1) {
-				 cerr << idl_global->prog_name()
+				 std::cerr << idl_global->prog_name()
 					  << GTDEVEL(": Could not remove cpp output file ")
 					  << tmp_file
 					  << "\n";
-			     exit(99);
+			     exit(EXIT_FAILURE);
 			}
 
 
-		    exit(99);
+		    exit(EXIT_FAILURE);
 		}
   }
   FILE * yyin = fopen(tmp_file, "r+");
   if (yyin == NULL) {
-    cerr << idl_global->prog_name()
+    std::cerr << idl_global->prog_name()
 	 << GTDEVEL(": Could not open cpp output file ")
 	 << tmp_file
 	 << "\n";
-    exit(99);
+    exit(EXIT_FAILURE);
   }
 
   (*DRV_FE_set_yyin)((File *) yyin);
@@ -574,11 +613,11 @@ DRV_pre_proc(char *myfile)
   }
   if (copy_src) {
     if (_unlink(tmp_ifile) == -1) {
-      cerr << idl_global->prog_name()
+      std::cerr << idl_global->prog_name()
 	   << GTDEVEL(": Could not remove cpp input file ")
 	   << tmp_ifile
 	   << "\n";
-      exit(99);
+      exit(EXIT_FAILURE);
     }
   }
 
@@ -591,14 +630,190 @@ DRV_pre_proc(char *myfile)
   fclose(yyin);
   if (_unlink((idl_global->temp_filename())->get_string()) == -1)
 	{
-	 cerr << idl_global->prog_name()
+	 std::cerr << idl_global->prog_name()
 	 << GTDEVEL(": Could not remove cpp output file ")
 	 << (idl_global->temp_filename())->get_string()
 	 << "\n";
-     exit(99);
+     exit(EXIT_FAILURE);
 	 }
   
     exit(0);
   }
 }
+#elif defined(__VMS)
+
+// OpenVMS Version
+
+#include <string>
+
+#if defined(__DECCXX) && __DECCXX_VER < 60000000 && !defined(std)
+// pretend we have an std namespace
+#define std
+#endif
+
+/*
+ * Pass input through preprocessor
+ */
+void
+DRV_pre_proc(char *myfile)
+{
+    int	wait_status;
+
+    long	readfromstdin = I_FALSE;
+
+    char	catbuf[512];
+
+
+    strcpy(tmp_file, "idlf_XXXXXX");
+    strcpy(tmp_ifile, "idli_XXXXXX");
+
+    (void) mktemp(tmp_file); strcat(tmp_file, ".cc");
+    (void) mktemp(tmp_ifile); strcat(tmp_ifile, ".cc");
+
+    if (strcmp(myfile, "standard input") == 0) {
+	idl_global->set_filename((*DRV_FE_new_UTL_String)(tmp_ifile));
+	idl_global->set_main_filename((*DRV_FE_new_UTL_String)(tmp_ifile));
+	idl_global->
+	    set_stripped_filename(
+		(*DRV_FE_new_UTL_String)(DRV_stripped_name(tmp_ifile))
+	    );
+	idl_global->set_real_filename((*DRV_FE_new_UTL_String)(tmp_ifile));
+	DRV_copy_input(stdin, tmp_ifile);
+	idl_global->set_read_from_stdin(I_TRUE);
+    } else {
+	if (copy_src) {
+	  FILE *fd = fopen(myfile, "r");
+	  DRV_copy_input(fd, tmp_ifile);
+	  fclose(fd);
+	}
+	idl_global->set_read_from_stdin(I_FALSE);
+	idl_global->set_filename((*DRV_FE_new_UTL_String)(myfile));
+	idl_global->set_main_filename((*DRV_FE_new_UTL_String)(myfile));
+	idl_global->
+	    set_stripped_filename(
+		(*DRV_FE_new_UTL_String)(DRV_stripped_name(myfile))
+	    );
+	if (copy_src) {
+	  idl_global->set_real_filename((*DRV_FE_new_UTL_String)(tmp_ifile));
+	}
+	else {
+	  idl_global->set_real_filename((*DRV_FE_new_UTL_String)(myfile));
+	}
+    }
+
+
+    {
+	if (copy_src)
+	  DRV_cpp_putarg(tmp_ifile);
+	else 
+	  DRV_cpp_putarg(myfile);
+	std::string commandLine(arglist[0]);
+	for (int i(1); arglist[i]; ++i) {
+	    std::string unixArg(arglist[i]);
+	    std::string vmsArg;
+
+	    if (unixArg[0]=='-') {
+		switch(unixArg[1]) {
+		    case 'E': {
+			vmsArg = std::string("/Prep=") + tmp_file;
+		    } break;
+		    case 'I': {
+			vmsArg = "/Incl=\"";
+			std::string includePath(unixArg.substr(2));
+			if (!includePath.empty() && includePath != ".") {
+			    vmsArg += includePath;
+			}
+			vmsArg += "\"";
+		    } break;
+		    case 'D': {
+			vmsArg = "/Defi=\"" + unixArg.substr(2) + "\"";
+		    } break;
+		    default: {
+			std::cerr << "Unix preprocessor switch: "
+			     << unixArg
+			     << " does not have corresponding VMS qualifier."
+			     << std::endl;
+		    }
+		}
+	    } else {
+		vmsArg = unixArg;
+	    }
+	    commandLine += " ";
+	    commandLine += vmsArg;
+	}
+
+	int spawn_rc = system(commandLine.c_str());
+	if (spawn_rc != 0 && spawn_rc & 1==0) {
+	    std::cerr << idl_global->prog_name() 
+		     << GTDEVEL(": command ")
+		     << commandLine
+		     << GTDEVEL(" failed\n");
+	    std::cerr << "Preprocessor returned non-zero status " << spawn_rc
+		 << std::endl;
+			
+	    if (copy_src) {
+		if (unlink(tmp_ifile) == -1) {
+		    std::cerr << idl_global->prog_name()
+			     << GTDEVEL(": Could not remove cpp input file ")
+			     << tmp_ifile
+			     << "\n";
+		    exit(EXIT_FAILURE);	
+		}
+	    }
+
+	    if (unlink(tmp_file) == -1) {
+		     std::cerr << idl_global->prog_name()
+			      << GTDEVEL(": Could not remove cpp output file ")
+			      << tmp_file
+			      << "\n";
+		 exit(EXIT_FAILURE);
+	    }
+
+
+	    exit(EXIT_FAILURE);
+	}   // system(...) failed
+    }	// scope for no apparent reason.
+    FILE * yyin = fopen(tmp_file, "r+");
+    if (yyin == NULL) {
+	std::cerr << idl_global->prog_name()
+	     << GTDEVEL(": Could not open cpp output file ")
+	     << tmp_file
+	     << "\n";
+	exit(EXIT_FAILURE);
+    }
+
+    (*DRV_FE_set_yyin)((File *) yyin);
+    if (idl_global->compile_flags() & IDL_CF_ONLY_PREPROC) {
+	std::string command(
+	    std::string("type ") + tmp_file
+	);
+	system(command.c_str());
+    }
+    if (copy_src) {
+	if (unlink(tmp_ifile) == -1) {
+	    std::cerr << idl_global->prog_name()
+	       << GTDEVEL(": Could not remove cpp input file ")
+	       << tmp_ifile
+	       << "\n";
+	    exit(EXIT_FAILURE);
+	}
+    }
+
+
+    idl_global->set_temp_filename((*DRV_FE_new_UTL_String)(tmp_file));
+
+    if (idl_global->compile_flags() & IDL_CF_ONLY_PREPROC) {
+	/* Remove the temporary file [containing the preprocessor output. */
+	fclose(yyin);
+	if (unlink((idl_global->temp_filename())->get_string()) == -1) {
+	    std::cerr << idl_global->prog_name()
+	    << GTDEVEL(": Could not remove cpp output file ")
+	    << (idl_global->temp_filename())->get_string()
+	    << "\n";
+	    exit(EXIT_FAILURE);
+	}
+  
+	exit(0);
+    }
+}
 #endif
