@@ -27,15 +27,47 @@
 
 /*
   $Log: o2be_array.cc,v $
+  Revision 1.13  1998/08/19 15:50:08  sll
+  New member functions void produce_binary_operators_in_hdr and the like
+  are responsible for generating binary operators <<= etc in the global
+  namespace.
+
+  Revision 1.12  1998/08/13 22:35:13  sll
+  Added pragma hdrstop to control pre-compile header if the compiler feature
+  is available
+
+  Revision 1.11  1998/04/07 18:39:39  sll
+  Use std::fstream instead of fstream.
+
+// Revision 1.10  1998/03/09  17:14:39  ewc
+// Use new _Forany function to get to underlying array slice. Avoid
+// explicit operator call that caused problems for aC++ on HPUX
+//
+// Revision 1.9  1998/01/27  16:33:34  ewc
+//  Added support for type any and TypeCode
+//
+  Revision 1.8  1997/12/23 19:27:51  sll
+  Bug fixes.
+
+  Revision 1.7  1997/12/18 17:28:43  sll
+  *** empty log message ***
+
+  Revision 1.6  1997/12/09 19:55:17  sll
+  *** empty log message ***
+
 // Revision 1.5  1997/05/06  13:49:08  sll
 // Public release.
 //
   */
 
 
-#include "idl.hh"
-#include "idl_extern.hh"
-#include "o2be.h"
+#include <idl.hh>
+#include <idl_extern.hh>
+#include <o2be.h>
+
+#ifdef HAS_pch
+#pragma hdrstop
+#endif
 
 #define ADPT_CLASS_TEMPLATE "_CORBA_Array_OUT_arg"
 
@@ -44,8 +76,9 @@ o2be_array::o2be_array(UTL_ScopedName *n,
 		       UTL_ExprList *dims)
   : AST_Array(n, ndims, dims),
     AST_Decl(AST_Decl::NT_array, n, NULL),
-    o2be_name(this)
+    o2be_name(AST_Decl::NT_array,n,NULL)
 {
+  set_recursive_seq(I_FALSE);
 }
 
 idl_bool
@@ -69,7 +102,9 @@ o2be_array::isVariable()
       break;
     case AST_Decl::NT_pre_defined:
       if (o2be_predefined_type::narrow_from_decl(decl)->pt() ==
-	  AST_PredefinedType::PT_any)
+	  AST_PredefinedType::PT_any || 
+	  o2be_predefined_type::narrow_from_decl(decl)->pt() == 
+	  AST_PredefinedType::PT_TypeCode)
 	isvar = I_TRUE;
       else
 	isvar = I_FALSE;
@@ -123,7 +158,7 @@ o2be_array::getNumOfElements()
 {
   size_t dim = 1;
   AST_Expression **d = dims();
-  int i;
+  unsigned long i;
   for (i=0; i < n_dims(); i++)
     {
       AST_Expression::AST_ExprValue *v = d[i]->ev();
@@ -240,7 +275,7 @@ dim_iterator::operator() ()
 }
 
 void
-o2be_array::produce_hdr (fstream &s, o2be_typedef *tdef)
+o2be_array::produce_hdr (std::fstream &s, o2be_typedef *tdef)
 {
   AST_Decl *decl = base_type();
 
@@ -252,18 +287,20 @@ o2be_array::produce_hdr (fstream &s, o2be_typedef *tdef)
       elm_fqname = o2be_string::fieldMemberTypeName();
       break;
     case AST_Decl::NT_interface:
-      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname(tdef);
       break;
     case AST_Decl::NT_typedef:
-      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname(tdef);
       break;
-#if USE_SEQUENCE_TEMPLATE_IN_PLACE
     case AST_Decl::NT_sequence:
-      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name();
+      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name(tdef);
       break;
-#endif
+    case AST_Decl::NT_pre_defined:
+      elm_fqname = o2be_predefined_type::narrow_from_decl(decl)->fieldMemberTypeName();
+      break;
+
     default:
-      elm_fqname = o2be_name::narrow_and_produce_fqname(decl);
+      elm_fqname = o2be_name::narrow_and_produce_unambiguous_name(decl,tdef);
       break;
     }
 
@@ -442,7 +479,7 @@ o2be_array::produce_hdr (fstream &s, o2be_typedef *tdef)
 }
 
 void
-o2be_array::produce_skel (fstream &s, o2be_typedef *tdef)
+o2be_array::produce_skel (std::fstream &s, o2be_typedef *tdef)
 {
   if (defined_in() == idl_global->root())
     {
@@ -519,33 +556,252 @@ o2be_array::produce_skel (fstream &s, o2be_typedef *tdef)
 }
 
 void
-o2be_array::produce_typedef_hdr(fstream &s, o2be_typedef *tdef1,
-				o2be_typedef *tdef2)
+o2be_array::produce_binary_operators_in_hdr(std::fstream &s,
+					    o2be_typedef *tdef)
 {
-  IND(s); s << "typedef " << tdef2->fqname() << " " << tdef1->uqname() << ";\n";
-  IND(s); s << "typedef " << tdef2->fqname() << "_slice " << tdef1->uqname() 
-	    << "_slice;\n";
-  IND(s); s << "typedef " << tdef2->fqname() << "_copyHelper "
-	    << tdef1->uqname() << "_copyHelper;\n";
-  IND(s); s << "typedef " << tdef2->fqname() << "_var "
-	    << tdef1->uqname() << "_var;\n";
-  IND(s); s << "typedef " << tdef2->fqname() << "_forany "
-	    << tdef1->uqname() << "_forany;\n";
-  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
-	    << tdef1->uqname() << "_slice* "<< tdef1->uqname() << "_alloc() "
-	    << "{ return " << tdef2->fqname() << "_alloc(); }\n";
-  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
-	    << tdef1->uqname() << "_slice* "<< tdef1->uqname() << "_dup(const "
-	    << tdef1->uqname() << "_slice* p) "
-	    << "{ return " << tdef2->fqname() << "_dup(p); }\n";
-  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
-	    << "void " << tdef1->uqname() << "_free( "
-	    << tdef1->uqname() << "_slice* p) "
-	    << "{ " << tdef2->fqname() << "_free(p); }\n\n";
+  if (idl_global->compile_flags() & IDL_CF_ANY)
+    {
+      if (check_recursive_seq() == I_FALSE)
+	{
+	  set_recursive_seq(I_FALSE);
+	  IND(s); s << "void operator<<=(CORBA::Any& _a, const " 
+		    << tdef->fqname() 
+		    << "_forany& _s);\n";
+	  IND(s); s << "CORBA::Boolean operator>>=(const CORBA::Any& _a, " 
+		    << tdef->fqname() << "_forany& _s);\n\n";
+    }
+    else set_recursive_seq(I_TRUE);
+  }
 }
 
 void
-o2be_array::_produce_member_decl (fstream &s, char *varname)
+o2be_array::produce_binary_operators_in_skel(std::fstream &s,
+					     o2be_typedef *tdef)
+{
+  if ((idl_global->compile_flags() &IDL_CF_ANY) && recursive_seq() == I_FALSE)
+    {
+      // Definitions of any insertion and extraction operators, and deletion
+      // function.
+
+      IND(s); s << "void _03RL_" << tdef->_fqname() 
+		<< "_delete(void* _data) {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << tdef->fqname() << "_slice* _0RL_t = (" << tdef->fqname() 
+		<< "_slice*) _data;\n";
+      IND(s); s << tdef->fqname() << "_free(_0RL_t);\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n\n";
+
+      IND(s); s << "void operator<<=(CORBA::Any& _a, const " << tdef->fqname() 
+		<< "_forany& _s) {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << "MemBufferedStream _0RL_mbuf;\n";
+      IND(s); s << tdef->fqtcname() << "->NP_fillInit(_0RL_mbuf);\n";
+    
+      {
+	o2be_operation::argMapping mapping;
+	o2be_operation::argType ntype = 
+	  o2be_operation::ast2ArgMapping(this,o2be_operation::wIN,mapping);
+	o2be_operation::produceMarshalCode(
+					   s,
+					   this,
+					   ScopeAsDecl(defined_in()),
+					   "_0RL_mbuf",
+					   "_s",
+					   ntype,
+					   mapping);
+      }
+
+      IND(s); s << "_a.NP_replaceData(" << tdef->fqtcname() << ",_0RL_mbuf);\n";
+      IND(s); s << tdef->fqname() << "_free(_s.NP_getSlice());\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n\n";    
+
+      IND(s); s << "CORBA::Boolean operator>>=(const CORBA::Any& _a, "
+		<< tdef->fqname() << "_forany& _s) {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << "CORBA::TypeCode_var _0RL_any_tc = _a.type();\n";
+      IND(s); s << "if (!_0RL_any_tc->NP_expandEqual(" << tdef->fqtcname() 
+		<< ",1)) {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << "return 0;\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n";
+      IND(s); s << "else {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << "void* _0RL_data = _a.NP_data();\n\n";
+      IND(s); s << "if (!_0RL_data) {\n";
+      INC_INDENT_LEVEL();
+      IND(s); s << "MemBufferedStream _0RL_tmp_mbuf;\n";
+      IND(s); s << "_a.NP_getBuffer(_0RL_tmp_mbuf);\n";
+      IND(s); s << tdef->fqname() << "_slice* _0RL_tmp = " << tdef->fqname()
+		<< "_alloc();\n";
+
+      {
+	o2be_operation::argMapping mapping;
+	o2be_operation::argType ntype = 
+	  o2be_operation::ast2ArgMapping(this,o2be_operation::wIN,mapping);
+	o2be_operation::produceUnMarshalCode(
+					     s,
+					     this,
+					     ScopeAsDecl(defined_in()),
+					     "_0RL_tmp_mbuf",
+					     "_0RL_tmp",
+					     ntype,
+					     mapping);
+      }
+
+      IND(s); s << "_0RL_data = (void*) _0RL_tmp;\n";
+      IND(s); s << "_a.NP_holdData(_0RL_data,_03RL_" << tdef->_fqname() 
+		<< "_delete);\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n\n";
+      IND(s); s << "_s = (" << tdef->fqname() << "_slice*) _0RL_data;\n";
+      IND(s); s << "return 1;\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n";
+      DEC_INDENT_LEVEL();
+      IND(s); s << "}\n\n";
+    }
+}
+
+
+void
+o2be_array::produce_typecode_skel(std::fstream &s)
+{
+  if (idl_global->compile_flags() & IDL_CF_ANY) {
+    // All array TypeCodes are generated in-place when they are used.
+    // Produce any static TypeCodes that are required by the array TypeCode
+    AST_Decl *decl = base_type();
+    if (!decl->in_main_file() || 
+	decl->node_type() == AST_Decl::NT_array || 
+	decl->node_type() == AST_Decl::NT_sequence)
+      o2be_name::narrow_and_produce_typecode_skel(decl,s);	       
+    }
+}
+
+void 
+o2be_array::produce_typecode_member(std::fstream &s, idl_bool new_ptr)
+{
+  if (idl_global->compile_flags() & IDL_CF_ANY) {
+    if (new_ptr) s << "new ";
+
+    AST_Expression **d = dims();
+    int count;
+    for(count = 0; count < n_dims(); count++)
+      {
+	s << "CORBA::TypeCode(CORBA::tk_array, ";
+	AST_Expression::AST_ExprValue *v = d[count]->ev();
+	switch (v->et) 
+	  {
+	  case AST_Expression::EV_short:
+	    s << v->u.sval;
+	    break;
+	  case AST_Expression::EV_ushort:
+	    s << v->u.usval;
+	    break;
+	  case AST_Expression::EV_long:
+	    s << v->u.lval;
+	    break;
+	  case AST_Expression::EV_ulong:
+	    s << v->u.ulval;
+	    break;
+	  default:
+	    throw o2be_internal_error(__FILE__,__LINE__,"unexpected type in array expression");
+	  }
+	s << ", ";
+      }    
+
+    AST_Decl *decl = base_type();
+    o2be_name::produce_typecode_member(decl,s,I_FALSE);
+
+    for (count = 0; count < n_dims(); count++) s << ")";
+  }
+}
+
+idl_bool 
+o2be_array::check_recursive_seq()
+{
+  AST_Decl *base_decl = base_type();
+  return o2be_name::narrow_and_check_recursive_seq(base_decl);
+}
+
+void
+o2be_array::produce_typedef_hdr(std::fstream &s, o2be_typedef *tdef1,
+				o2be_typedef *tdef2)
+{
+  IND(s); s << "typedef " << tdef2->unambiguous_name(tdef1) 
+	    << " " << tdef1->uqname() << ";\n";
+  IND(s); s << "typedef " << tdef2->unambiguous_name(tdef1) 
+	    << "_slice " << tdef1->uqname() 
+	    << "_slice;\n";
+  IND(s); s << "typedef " << tdef2->unambiguous_name(tdef1) 
+	    << "_copyHelper " << tdef1->uqname() << "_copyHelper;\n";
+  IND(s); s << "typedef " << tdef2->unambiguous_name(tdef1) 
+	    << "_var " << tdef1->uqname() << "_var;\n";
+  IND(s); s << "typedef " << tdef2->unambiguous_name(tdef1) 
+	    << "_forany " << tdef1->uqname() << "_forany;\n";
+  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
+	    << tdef1->uqname() << "_slice* "<< tdef1->uqname() << "_alloc() ";
+  if (!(tdef1->defined_in()==idl_global->root())) {
+    s << "{ return " << tdef2->unambiguous_name(tdef1) << "_alloc(); }\n";
+  }
+  else {
+    s << ";\n";
+  }
+  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
+	    << tdef1->uqname() << "_slice* "<< tdef1->uqname() << "_dup(const "
+	    << tdef1->uqname() << "_slice* p) ";
+  if (!(tdef1->defined_in()==idl_global->root())) {
+    s << "{ return " << tdef2->unambiguous_name(tdef1) 
+      << "_dup(p); }\n";
+  }
+  else {
+    s << ";\n";
+  }
+  IND(s); s << ((!(tdef1->defined_in()==idl_global->root()))?"static ":"extern ")
+	    << "void " << tdef1->uqname() << "_free( "
+	    << tdef1->uqname() << "_slice* p) ";
+  if (!(tdef1->defined_in()==idl_global->root())) {
+    s << "{ " << tdef2->unambiguous_name(tdef1) 
+      << "_free(p); }\n\n";
+  }
+  else {
+    s << ";\n\n";
+  }
+}
+
+void
+o2be_array::produce_typedef_skel(std::fstream &s, o2be_typedef *tdef1,
+				 o2be_typedef *tdef2)
+{
+  if (tdef1->defined_in() == idl_global->root()) {
+    IND(s); s << "extern " << tdef1->fqname() << "_slice* " 
+	      << tdef1->fqname() << "_alloc() {\n";
+    INC_INDENT_LEVEL();
+    IND(s); s << "return " << tdef2->fqname() << "_alloc();\n";
+    DEC_INDENT_LEVEL();
+    IND(s); s << "}\n\n";
+
+    IND(s); s << "extern " << tdef1->fqname() << "_slice* " 
+	      << tdef1->fqname() << "_dup(const " << tdef1->fqname()
+	      << "_slice* p) {\n";
+    INC_INDENT_LEVEL();
+    IND(s); s << "return " << tdef2->fqname() << "_dup(p);\n";
+    DEC_INDENT_LEVEL();
+    IND(s); s << "}\n\n";
+
+    IND(s); s << "extern void " << tdef1->fqname() << "_free( " 
+	      << tdef1->fqname() << "_slice* p) {\n";
+    INC_INDENT_LEVEL();
+    IND(s); s << tdef2->fqname() << "_free(p);\n";
+    DEC_INDENT_LEVEL();
+    IND(s); s << "}\n\n";
+  }
+}
+
+void
+o2be_array::_produce_member_decl (std::fstream &s, char *varname,AST_Decl* used_in)
 {
   AST_Decl *decl = base_type();
 
@@ -557,18 +813,20 @@ o2be_array::_produce_member_decl (fstream &s, char *varname)
       elm_fqname = o2be_string::fieldMemberTypeName();
       break;
     case AST_Decl::NT_interface:
-      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname(used_in);
       break;
     case AST_Decl::NT_typedef:
-      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname(used_in);
       break;
-#if USE_SEQUENCE_TEMPLATE_IN_PLACE
     case AST_Decl::NT_sequence:
-      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name();
+      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name(used_in);
+      break;
+    case AST_Decl::NT_pre_defined:
+      elm_fqname = o2be_predefined_type::narrow_from_decl(decl)->fieldMemberTypeName();
       break;
-#endif
     default:
-      elm_fqname = o2be_name::narrow_and_produce_fqname(decl);
+      elm_fqname = o2be_name::narrow_and_produce_unambiguous_name(decl,
+								  used_in);
       break;
     }
 
@@ -605,23 +863,27 @@ o2be_array::_produce_member_decl (fstream &s, char *varname)
 }
 
 void
-o2be_array::produce_struct_member_decl (fstream &s, AST_Decl *fieldtype)
+o2be_array::produce_struct_member_decl (std::fstream &s, AST_Decl *fieldtype,
+					AST_Decl* used_in)
 {
-  _produce_member_decl(s,o2be_name::narrow_and_produce_uqname(fieldtype));
+  _produce_member_decl(s,o2be_name::narrow_and_produce_uqname(fieldtype),
+		       used_in);
 }
 
 void
-o2be_array::produce_union_member_decl (fstream &s, AST_Decl *fieldtype)
+o2be_array::produce_union_member_decl (std::fstream &s, AST_Decl *fieldtype,
+				       AST_Decl* used_in)
 {
   char * varname = new char[strlen("pd_")+strlen(o2be_name::narrow_and_produce_uqname(fieldtype))+1];
   strcpy(varname,"pd_");
   strcat(varname,o2be_name::narrow_and_produce_uqname(fieldtype));
-  _produce_member_decl(s,varname);
+  _produce_member_decl(s,varname,used_in);
   delete [] varname;
 }
 
 void
-o2be_array::produce_typedef_in_union(fstream &s, const char *tname)
+o2be_array::produce_typedef_in_union(std::fstream &s, const char *tname,
+				     AST_Decl* used_in)
 {
   AST_Decl *decl = base_type();
 
@@ -633,22 +895,24 @@ o2be_array::produce_typedef_in_union(fstream &s, const char *tname)
       elm_fqname = o2be_string::fieldMemberTypeName();
       break;
     case AST_Decl::NT_interface:
-      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_interface::narrow_from_decl(decl)->fieldMemberType_fqname(used_in);
       break;
     case AST_Decl::NT_typedef:
-      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname();
+      elm_fqname = o2be_typedef::narrow_from_decl(decl)->fieldMemberType_fqname(used_in);
       break;
-#if USE_SEQUENCE_TEMPLATE_IN_PLACE
     case AST_Decl::NT_sequence:
-      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name();
+      elm_fqname = o2be_sequence::narrow_from_decl(decl)->seq_template_name(used_in);
+      break;
+    case AST_Decl::NT_pre_defined:
+      elm_fqname = o2be_predefined_type::narrow_from_decl(decl)->fieldMemberTypeName();
       break;
-#endif
     default:
-      elm_fqname = o2be_name::narrow_and_produce_fqname(decl);
+      elm_fqname = o2be_name::narrow_and_produce_unambiguous_name(decl,
+								  used_in);
       break;
     }
 
-  IND(s); s << "typedef " << elm_fqname << " " << tname;
+  IND(s); s << "typedef " << elm_fqname << " _0RL" << tname;
   {
     int nd = n_dims();
     AST_Expression **d = dims();
@@ -714,17 +978,26 @@ o2be_array::produce_typedef_in_union(fstream &s, const char *tname)
 }
 
 const char*
-o2be_array::out_adptarg_name(o2be_typedef* tdef) const
+o2be_array::out_adptarg_name(o2be_typedef* tdef,AST_Decl* used_in) const
 {
+  const char* ubname;
+
+  if (o2be_global::qflag()) {
+    ubname = tdef->fqname();
+  }
+  else {
+    ubname = tdef->unambiguous_name(used_in);
+  }
+
   char* p = new char[strlen(ADPT_CLASS_TEMPLATE)+strlen("<, >")+
-		     strlen(tdef->fqname()) + strlen("_slice") +
-		     strlen(tdef->fqname()) + strlen("_var")+1];
+		     strlen(ubname) + strlen("_slice") +
+		     strlen(ubname) + strlen("_var")+1];
   strcpy(p,ADPT_CLASS_TEMPLATE);
   strcat(p,"<");
-  strcat(p,tdef->fqname());
+  strcat(p,ubname);
   strcat(p,"_slice");
   strcat(p,",");
-  strcat(p,tdef->fqname());
+  strcat(p,ubname);
   strcat(p,"_var >");
   return p;  
 }
