@@ -25,6 +25,13 @@
 
 /*
   $Log: o2be_field.cc,v $
+  Revision 1.4  1998/08/13 22:42:42  sll
+  Added pragma hdrstop to control pre-compile header if the compiler feature
+  is available.
+
+  Revision 1.3  1997/12/09 19:55:39  sll
+  *** empty log message ***
+
 // Revision 1.2  1997/05/06  13:58:00  sll
 // Public release.
 //
@@ -34,10 +41,14 @@
 #include <idl_extern.hh>
 #include <o2be.h>
 
+#ifdef HAS_pch
+#pragma hdrstop
+#endif
+
 o2be_field::o2be_field(AST_Type *ft, UTL_ScopedName *n, UTL_StrList *p)
 	: AST_Field(ft, n, p),
 	  AST_Decl(AST_Decl::NT_field, n, p),
-	  o2be_name(this)
+	  o2be_name(AST_Decl::NT_field,n,p)
 {
 }
 
